# -*- coding: utf-8 -*-
"""
Created on Thu Jun  1 11:11:57 2017

@author: hector
"""
import datetime
import time
import logging


class SensorInfo:
    """
      cdirtry - True if campaign info is outdated
      sdirty - True if system information is outdated
      idirty - True if software information is outdated
     """

    def __init__(self, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.timeAtSensor = None
        self.timeAtController = None
        self.status = "OFF"
        self.name = "UnknownName"
        self.model = "UnknownModel"
        self.serial = "0"
        self.campaign = None
        self.statusInfo = {}
        self.lastError = ""
        self.cdirty = False
        self.sdirty = False
        self.idirty = False

    def addStatus(self, serial, timestamp, uptime, status, statusInfo):
        self.serial = serial
        # All timestamps considered as int seconds from epoch
        self.timeAtSensor = timestamp
        self.uptime = uptime
        self.status = status

        # Error conditions handling
        # Should be cleaner, maybe managing status better at sensor
        # Don't flush the last known error in any case, better for debugging
        if statusInfo.get("Status", "") == "ERROR":
            self.lastError = statusInfo.get("Cause", "Unknown error")
        else:
            if not status == "BUSY":
                self.statusInfo = statusInfo
        self.updateControllerTime()
        self.sdirty = True

    def addInfo(self, serial, timestamp, sensorType, softwareInfo):
        self.serial = serial
        self.timeAtSensor = timestamp
        self.sensorType = sensorType
        self.softwareInfo = softwareInfo
        self.updateControllerTime()
        self.idirty = True

    def updateControllerTime(self):
        self.timeAtController = int(
                time.mktime(datetime.datetime.now().timetuple()))
        if self.status != "OFF" and self.timeOffset() > 10:
            self.logger.warn("Time delta at sensor %s is important.",
                             self.serial)

    def setCampaign(self, campaign):
        if campaign == self.campaign:
            return
        self.campaign = campaign
        self.cdirty = True

    def getCampaign(self):
        if self.campaign is None:
            return "None"
        return self.campaign

    def setGroups(self, groups):
        self.groups = groups
        self.dirty = True

    def timeOffset(self):
        if self.timeAtSensor is None or self.timeAtController is None:
            return 0
        timeDiff = self.timeAtController - self.timeAtSensor
        return timeDiff
